#include <iostream>
#include <utility>
#include <algorithm>
#include <boost/bind.hpp>
#include <boost/graph/graph_traits.hpp>
#include <boost/graph/adjacency_list.hpp>

using namespace boost;
using namespace std;

//Szablon klasy dostarczajacej informacji o grafie
template<class Graph> struct Show {
	Graph& g_;

	Show(Graph& g) : g_(g) {}

	typedef typename graph_traits<Graph>::vertex_descriptor Vertex;
	typedef typename graph_traits<Graph>::edge_descriptor Edge;
	typedef typename property_map<Graph, vertex_index_t>::type VertexIndexType;

	void showVertexId(const Vertex& v) const {
		VertexIndexType vertex_id = get(vertex_index, g_);
		cout << get(vertex_id, v);
	}
	void showEdgeIds(const Edge& e) const {
		Vertex src = source(e, g_);
		Vertex tar = target(e, g_);
		cout << '('; showVertexId(src); cout << ','; showVertexId(tar); cout << ')';
	}
	void showVertexAll(const Vertex& v) const {
		cout << "id: "; showVertexId(v);
		cout << " out edges: ";
		for_each( out_edges(v, g_).first, out_edges(v, g_).second,
				  bind(&Show::showEdgeIds, this, _1) );
		cout << " in edges: ";
		for_each( in_edges(v, g_).first, in_edges(v, g_).second,
				  bind(&Show::showEdgeIds, this, _1) );
		cout << "out vertexs: ";
		for_each( adjacent_vertices(v, g_).first, adjacent_vertices(v, g_).second,
				  bind(&Show::showVertexId, this, _1) );
		cout << endl;
	}
};

int main() {
    typedef adjacency_list<vecS, vecS, bidirectionalS> Graph;     //typ grafu

    enum { A, B, C, D, N };               // identyfikatory wierzcholkow
    const int NUM_VERTICES = N;

    typedef std::pair<int, int> Edge;
    const Edge EDGE_ARRAY[] =             // krawedzie dla grafu
		{ Edge(A,B), Edge(A,C),
		  Edge(B,A), Edge(B,D),
		  Edge(C,B), Edge(D,C) };
    const int NUM_EDGES = sizeof(EDGE_ARRAY)/sizeof(EDGE_ARRAY[0]);
	//tworzy graf
    Graph g(EDGE_ARRAY, EDGE_ARRAY + NUM_EDGES, NUM_VERTICES);

	Show<Graph> show(g);
	//przeglada wierzcholki grafu
	std::for_each( vertices(g).first, vertices(g).second,
				   bind(&Show<Graph>::showVertexAll, show, _1) );
	//przeglada krawedzie dla grafu
	std::for_each( edges(g).first, edges(g).second,
				   bind(&Show<Graph>::showEdgeIds, show, _1) );
    return 0;
}

